import os
import sys
import time

from gettext import gettext as _

from fiveaday import files, lpteams

try:
    from bzrlib import branch, errors
except:
    print >> sys.stderr, _("You need to install the  bzr  package.")
    sys.exit(1)

from core import Core
c = Core()

def add_if_not_in_inventory(branch_dir, filename):
    f = os.path.basename(filename)
    b = branch.Branch.open(branch_dir)
    last_rev = b.last_revision()
    if not last_rev or last_rev == "null:":
        return os.system("cd %s; bzr add %s" % (branch_dir, filename))
    
    inv = b.repository.get_inventory(b.last_revision())
    if f not in [x[0] for x in inv.iter_entries()]:
        return os.system("cd %s; bzr add %s" % (branch_dir, filename))
    return 0

def setup_local_branch():
    path = c.local_branch
    legacy_path = os.path.expanduser("~/.5-a-day-data")
    legacy_file = os.path.join(legacy_path, c.lplogin)
    legacy_team_file = os.path.expanduser("~/.5-a-day-team")
    err = 0
    if not os.path.exists(path):
        os.makedirs(path)
        os.system("bzr init %s" % path)
        if os.path.exists(legacy_file):
            os.system("cp %s %s" % (legacy_file, c.log_file))
            os.system("cp %s %s" % (legacy_team_file, c.team_file))
            add_if_not_in_inventory(c.local_branch, 
                                    os.path.basename(c.log_file))
            add_if_not_in_inventory(c.local_branch, 
                                    os.path.basename(c.team_file))
        else:
            os.system("touch %s" % c.log_file)
            add_if_not_in_inventory(c.local_branch, 
                                    os.path.basename(c.log_file))
        os.system("cd %s; bzr commit -m 'change to new branch layout'" % c.local_branch)
        err = os.system("cd %s; bzr push --remember %s" % (c.local_branch, c.remote_branch))
        if not err:
            print _("You can now safely remove ~/.5-a-day-data")
            if os.path.exists(legacy_team_file):
                print _("You can now safely remove ~/.5-a-day-team")
    return err

def checkout_branch():
    try:
        branch.Branch.open_containing(c.remote_readonly_branch)
    except errors.NotBranchError:
        err = setup_local_branch()
        return err
    err = os.system("bzr branch %s %s" % (c.remote_readonly_branch, c.local_branch))
    return err

def update_branch():
    if not os.path.exists(c.local_branch):
        err = checkout_branch()
        if err: return err #if unable to checkout the branch return bzr-error
    cwd = os.getcwd()
    err = os.system("cd %s; bzr pull %s" % (c.local_branch, c.remote_branch))
    os.system("cd %s" % cwd) #always cd back to working-dir, this should not fail
    return err #return 0 if 'bzr update' was successfull or the bzr-error otherwise

def commit_changes(force_push):
# Make sure there is something to commit otherwise "bzr commit" will
# fail, if there's nothing to commit.
    import subprocess
    if not subprocess.Popen(["bzr", "status", c.local_branch], 
                            stdout=subprocess.PIPE).communicate()[0]:
        return 0

    err = os.system("cd %s; bzr commit -m 'updated log for %s'" % (c.local_branch, c.lplogin))
    if err:
        if err == 768:
            err = os.system("cd %s; bzr merge" % c.local_branch)
            if err:
                # stop if merging branch was not successful
                print >> sys.stderr, _("bzr failed with error code"), err
                return err
            os.system("cd %s; bzr commit -m 'merged log for %s'" % (c.local_branch, c.lplogin))
        else:
            # stop if creating/updating branch was not successful
            print >> sys.stderr, _("bzr failed with error code"), err
            return err

    # check if pushed already in the last hour:
    if not force_push:
        try:
            b = branch.Branch.open(c.remote_readonly_branch)
            last_rev = b.last_revision()
        except errors.NotBranchError:
            last_rev = None
        if last_rev:
            last_rev = b.repository.get_revision(last_rev)
            seconds_since_last_commit = time.time() - last_rev.timestamp
            if seconds_since_last_commit < 3600:
                print >> sys.stdout, _("Bugs have been added, but not committed yet (already committed %d minutes ago).") % (seconds_since_last_commit/60)
                return 0 # This is no error 

    err = update_branch()
    if err:
        # stop if creating/updating branch was not successful
        print >> sys.stderr, _("bzr failed with error code"), err
        return 103 #103: error while updating bzr branch

    err = os.system("cd %s; bzr push --remember %s" % (c.local_branch, c.remote_branch))
    return err #return 0 if 'bzr push' was successfull or the bzr-error otherwise

def add(bugs, force_push=False):
    log = files.LogFile()

    err = log.add(time.time(), bugs)
    if err:
        return err
    if log.tags:
        print _("'%s' tagged as '%s'.") % (" ". join(bugs), 
                                           " ". join(log.tags))
    err = commit_changes(force_push)
    if err:
        print >> sys.stderr, _("bzr failed with error code"), err
        return 104 #104: error while committing changes
    return 0
