import os
import sys
import time

from gettext import gettext as _

import lpteams

from core import Core
c = Core()

def get_tags():
    if os.path.exists(c.tag_file):
        return map(lambda a: a.strip(), open(c.tag_file).readlines())
    return []

def get_teams():
    if os.path.exists(c.team_file):
        return map(lambda a: a.strip(), open(c.team_file).readlines())
    return []

def preserve_old_signature():
    sig = c.signature_file
    oldsig = c.old_signature_file
    if os.path.exists(sig):
        if not open(sig).read().count("https://wiki.ubuntu.com/5-A-Day") and \
           not os.path.exists(oldsig):
            os.system("mv %s %s" % (sig, oldsig))
        else:
            os.remove(sig)

def get_old_log():
    if os.path.exists(c.log_file):
        lines = open(c.log_file).readlines()
        return filter(lambda b: b != "", map(lambda a: a.strip(), lines))
    else:
        return []

def first_element_is_int(s):
    a = s.split()[0]
    try:
        i = int(a)
        return True
    except:
        return False

class LogFile(object):
    def __init__(self):
        self.tags = get_tags()
        self.entries = get_old_log()
        if self.entries:
            # we used "2008-4-1 123456" entries before, now they're simply
            # <timestamp> <bug>
            self.correct_new_entries = filter(lambda a: first_element_is_int(a),
                                              self.entries)
        else:
            self.correct_new_entries = []

    def add(self, t, entries):
        # TODO: move parsing/validation of bug numbers to parser?!
        #       This method should only get a list of bug numbers probably.

        # Parse/validate bug arguments:
        bugs = []
        for entry in entries:
            for i in entry.split(","):
                n = i.strip(" #[]()")
                if n:
                    try:
                        number = int(n)
                    except:
                        print >> sys.stderr, _("'%s' is not a valid bug number. Aborting.") % n
                        return 105 #105: invalid bug number
                    if filter(lambda a: (int(t)-int(a.split()[0]) < 60*60*24) and \
                                        (a.split()[1]==str(n)), 
                              self.correct_new_entries):
                        print >> sys.stderr, _("Bug '%s' has already been added today. Aborting.") % n
                        return 106 #106: bug has already been added today
                    bugs.append(n)
        # Write bugs to file:
        if os.path.exists(c.log_file):
            f = open(c.log_file, "a")
        else:
            f = open(c.log_file, "w")
        for bug in bugs:
            data_str = "%s %s %s\n" % (int(t), bug, " ".join(self.tags))
            f.write(data_str)
        f.close()

def add_tags(tags):
    current_tags = get_tags()
    for t in tags:
        if t not in current_tags:
            current_tags.append(t)
    set_tags(current_tags)

def remove_tags(tags):
    current_tags = get_tags()
    for t in tags:
        if t in current_tags:
            current_tags.remove(t)
    set_tags(current_tags)

def set_tags(new_tags):
    f = open(c.tag_file, "w")
    f.write("\n".join(new_tags))
    f.close()

def show_tags():
    tags = get_tags()
    if not tags:
        print _("No tags set right now.")
    else:
        if len(tags) == 1:
            print _("Your current tag is: %s" % tags[0])
        else:
            print _("Your current tags are: %s" % ", ".join(tags))

def set_teams(new_teams):
    current_teams = get_teams()
    try:
        os.remove(c.team_file)
    except:
        pass

    lp_teams = [a for a in lpteams.get_teams(c.lplogin)]
    for t in new_teams:
        if t not in current_teams:
            if t in lp_teams:
                current_teams.append(t)
            else:
                print >> sys.stderr, _("You're not in team '%s', not adding.") % t
    for t in current_teams:
        if t not in new_teams:
            current_teams.remove(t)
        if t not in lp_teams:
            current_teams.remove(t)

    f = open(c.team_file, "w")
    f.write("\n".join(current_teams))
    f.close()
    return True

def add_teams(teams):
    current_teams = get_teams()
    for t in teams:
        if t not in current_teams:
            current_teams.append(t)
    set_teams(current_teams)

def remove_teams(teams):
    current_teams = get_teams()
    for t in teams:
        if t in current_teams:
            current_teams.remove(t)
    set_teams(current_teams)

def show_teams():
    teams = get_teams()
    if not teams:
        print _("No teams set right now.")
    else:
        if len(teams) == 1:
            print _("Your current team is: %s" % teams[0])
        else:
            print _("Your current teams are: %s" % ", ".join(teams))

