#    5-a-day - utilities to participate in 5-A-Day statistics
#    Copyright (C) 2008  Canonical Ltd.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gettext import gettext as _
import getopt
import sys

# error codes
COMMAND_LINE_SYNTAX_ERROR = 2


def usage():
    # TODO: update usage string
    print _("""5-a-day --add <bug numbers>        add <bug numbers> to your 5-a-day
    -f | --force                   force commit to the remote branch

5-a-day --update                   update ~/.signature with the bugs you've worked on
    --html                         update ~/.signature and print HTML signature
    --yesterday                    update ~/.signature with yesterdays bugs
    
5-a-day --add-tag(s) <tag(s)>      tag subsequent bugs with <tag>
5-a-day --remove-tag(s) <tag(s)>   remove <tag> from your tag list
5-a-day --show-tag(s)              show current tag(s)""")


# TODO: Option "-n"/"--no-remote" to never update/commit the branch


def parse_args():
    try:
        long_opts = [
            "add",
            "add-tag",
            "add-tags",
            "add-team",
            "add-teams",
            "force",
            "help",
            "html",
            "remove-tag",
            "remove-tags",
            "remove-team",
            "remove-teams",
            "show-tag",
            "show-tags",
            "show-team",
            "show-teams",
            "update",
            "verbose",
            "yesterday"
        ]
        opts, args = getopt.gnu_getopt(sys.argv[1:], "afhuv", long_opts)
    except getopt.GetoptError, e:
        # print help information and exit:
        print str(e) # will print something like "option -a not recognized"
        usage()
        sys.exit(COMMAND_LINE_SYNTAX_ERROR)

    commands = set()
    flags = {
        "force": False,
        "html": False,
        "show-tags": False,
        "show-teams": False,
        "update": False,
        "verbose": False,
        "yesterday": False,
    }

    for o, a in opts:
        if o in ("-a", "--add"):
            commands.add("add")
        elif o in ("--add-tag", "--add-tags"):
            commands.add("add-tags")
        elif o in ("--add-team", "--add-teams"):
            commands.add("add-teams")
        elif o in ("-f", "--force"):
            flags["force"] = True
        elif o in ("-h", "--help"):
            usage()
            sys.exit(0)
        elif o in ("--html"):
            flags["html"] = True
        elif o in ("--remove-tag", "--remove-tags"):
            commands.add("remove-tags")
        elif o in ("--remove-team", "--remove-teams"):
            commands.add("remove-teams")
        elif o in ("--show-tag", "--show-tags"):
            flags["show-tags"] = True
        elif o in ("--show-team", "--show-teams"):
            flags["show-teams"] = True
        elif o in ("-u", "--update"):
            flags["update"] = True
        elif o in ("-v", "--verbose"):
            flags["verbose"] = True
        elif o in ("--yesterday"):
            flags["yesterday"] = True
        else:
            assert False, _("unhandled option")

    if len(commands) == 1 and len(args) == 0:
        print _("E: You have to specify at least one argument.")
        sys.exit(COMMAND_LINE_SYNTAX_ERROR)

    if len(commands) > 1:
        print _("E: You must not specify more than one of those tags: --add,"
                " --add-tag, --add-team, --remove-tag, --remove-team")
        return COMMAND_LINE_SYNTAX_ERROR

    # data belongs to --add, --add-tag, --add-team, --remove-tag or --remove-team
    return commands, flags, args
