import gobject
import re
import subprocess
import os.path
import sys
import threading

import gtk

sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
from fiveaday import lpteams, files
import fiveadayapplet

from gettext import gettext as _

from core import Core
c = Core()

URL_TO_NUM = re.compile(r'^.+\+bug\/([0-9]+).*$')

def commit(bugnumber, tool):
    #TODO: put this in a seperate thread !?
    # replace this by fiveaday.bzr.add()??
    if not os.path.exists(tool):
        return 101 #101: 5-a-day not found
    s = subprocess.Popen((tool,"--add",str(bugnumber)), stderr=subprocess.PIPE, stdout=subprocess.PIPE)
    x = s.wait()
    if x != 0:
        print >> sys.stderr, _("bzr failed with an error. Debug information:"),\
            s.stdout.read() or None, s.stdout.read() or None
    return x
    
def get_teams_data(user):
    all_teams = dict((i,False) for i in lpteams.get_teams(user))
    for i in c.lplogin:
        if not i in all_teams:
            raise KeyError, _("You are not a member of '%s'") %i
        all_teams[i] = True
    return all_teams
    
def get_username():
    return c.lplogin
    
class CommitThread(threading.Thread):
    def __init__(self, bugnr, tool, view):
        super(CommitThread, self).__init__()
        self.view = view
        self.bugnr = bugnr
        self.tool = tool
        self.quit = False

    def update_tooltip(self, message):
        self.view.change_tooltip(message)
        return False

    def run(self):
        nr = self.bugnr
        save_tooltip = self.view.applet.get_tooltip_text()
        self.update_tooltip(_("Committing Bug #%s") %nr)
        print >>sys.stderr, "Committing Bug #%s ..." %nr
        err = commit(nr, self.tool)
        print >>sys.stderr, "... Finished with ErrCode", err
        if err == 102:
            print >>sys.stderr, "Open login dialog"
            x = self.view.do_login_show()
            print >>sys.stderr, "Result", x
            if x:
                f = file(os.path.expanduser(fiveadayapplet.CONFIG_FILE), 'w')
                f.write(x)
                f.close()
                print >>sys.stderr, "Committing Bug #%s ..." %nr
                err = commit(nr, self.tool)
                print >>sys.stderr, "... Finished with ErrCode", err
                if err:
                    print >>sys.stderr, "unknown error while running '5-a-day', can't commit bug"
                    text = "unknown error while running '5-a-day',\ncan't commit Bug #%s" %nr
                    self.view.show_notification("5-a-day", text)
                self.update_tooltip(save_tooltip)
                return
            print >>sys.stderr, "No LP-login given"
            text = "unknown error while running '5-a-day',\ncan't commit Bug #%s" %nr
            self.view.show_notification("5-a-day", text)
        elif err:
            err_data = {
                101: _("Error while submitting Bug #%s:\n'5-a-day' not found") %nr,
                106: _("Error while submitting Bug #%s:\nThis Bug has already been added today") %nr}
            if err in err_data:
                text = err_data[err]
            else:
                text = _("Unknown error while submitting Bug #%s:\nError-code: %s") %(nr, err)
            self.view.show_notification("5-a-day", text)
        else:
            text = _("Bug #%s successfully submitted") %nr
            tags = files.get_tags()
            if tags:
                text += "\n<b>Tags: %s</b>" %", ".join(tags)
            self.view.show_notification("5-a-day", text)
        self.update_tooltip(save_tooltip)



class Controller(object):
    def __init__(self, applet, data_dir, dev_mode=False):
        if dev_mode:
            self.tool = "%s/5-a-day" %dev_mode
        else:
            self.tool = "/usr/bin/5-a-day"
        self.view = fiveadayapplet.View(self, applet, data_dir)
        self.user = get_username()
        if self.user:
            self.model = fiveadayapplet.Model(self, get_teams_data(self.user))
        else:
            self.model = fiveadayapplet.Model(self)
        self.view.setup_treeview(self.model.get_treemodel())
        
        self.tags_model = fiveadayapplet.TagsModel(self, files.get_tags())
        self.view.setup_tags_treeview(self.tags_model.get_treemodel())
        
        
    def on_menuitem_show_wiki_activate(self, *_):
        self.view.do_wiki_open()
        
    def on_menuitem_show_stats_activate(self, *_):
        self.view.do_stats_open()
        
    def on_menuitem_about_activate(self, *_):
        self.view.do_about_show()
        
    def on_menuitem_tags_activate(self, *_):
        x = self.view.do_tags_show()
        if x:
            f = open(c.tag_file, "w")
            f.write("\n".join(self.tags_model.values))
            f.close()
            
        
    def on_menuitem_teams_activate(self, *_):
        if not self.user:
            x = self.view.do_login_show()
            if x:
                self.user = x
                self.model.add_teams(get_teams_data(self.user))
                f = file(os.path.expanduser(fiveadayapplet.CONFIG_FILE), 'w')
                f.write(x)
                f.close()
        if not self.user:
            return
        
        x = self.view.do_teams_show()
        if x:
            files.set_teams(x)
        
    # Events
        
    def on_applet_click(self, _, event):
        if event.button == 1:
            pass

    def on_drag_data_received(self, widget, context, x, y, selection, target_type, timestamp):
        if target_type == fiveadayapplet.TARGET_TYPE_URI_LIST:
            data = selection.data.strip()
            url = data.split("\n", 1)[0]
            m = URL_TO_NUM.search(url)
            if m:
                nr = int(m.group(1))
                t = CommitThread(nr, self.tool, self.view)
                gtk.gdk.threads_enter()
                t.start()
                gtk.gdk.threads_leave()
            else:
                text = _("Invalid content, nothing to commit")
                self.view.show_notification("5-a-day", text)
        else:
            text = _("Invalid data format, nothing to commit")
            self.view.show_notification("5-a-day", text)
