import os
import gtk
import gtk.glade
import pynotify

from gettext import gettext as _

import fiveadayapplet


class View(object):

    def __init__(self, controller, applet, data_dir):
        
        pynotify.init("5-a-day Applet")
        
        self.controller = controller
        self.applet = applet
        self.applet.set_background_widget(applet)
        self.xml = gtk.glade.XML(os.path.join(data_dir, '5-a-day-applet.glade'))
        
        self.change_tooltip(_("Drag & Drop bugs over this applet"))

        self.icon_filename = os.path.join(data_dir, '5-a-day.svg')
        self.about_icon = gtk.gdk.pixbuf_new_from_file(self.icon_filename)

        if self.applet:

            evbox = gtk.EventBox()
            evbox.connect('drag_data_received', controller.on_drag_data_received)
            evbox.drag_dest_set( gtk.DEST_DEFAULT_MOTION |
                        gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP,
                        fiveadayapplet.dnd_list, gtk.gdk.ACTION_COPY)

            evbox.set_visible_window(False)
            #~ evbox.connect("button-press-event", controller.on_applet_click)

            size = int(min(self.applet.get_size() * 0.9, 100))
            pb = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(data_dir, '5-a-day.svg'), size, size)
            img = gtk.image_new_from_pixbuf(pb)

            evbox.add(img)
            self.applet.add(evbox)
            self.applet.show_all()

            propxml = """
            <popup name="button3">
                <menuitem name="Show Wiki" verb="show_wiki" pixtype="stock"
                    pixname="gtk-help" label="%(show_wiki)s"/>
                <menuitem name="Show stats" verb="show_stats" label="%(show_stats)s" pixtype="stock" pixname="gtk-dialog-info"/>
                <menuitem name="Show teams" verb="show_teams" label="%(show_teams)s"/>
                <menuitem name="Show tags" verb="show_tags" label="%(show_tags)s"/>
                <menuitem name="About" verb="show_about" _label="_About" pixtype="stock" pixname="gtk-about" />
            </popup>
            """ % {
                'show_wiki' : _("Open _Wiki page"),
                'show_teams' : _("Manage LoCo _Teams"),
                'show_stats' : _("Open _Stats"),
                'show_tags' : _("_Manage Tags"),
            }

            self.applet.setup_menu(propxml, (
                ("show_wiki", self.controller.on_menuitem_show_wiki_activate),
                ("show_stats", self.controller.on_menuitem_show_stats_activate),
                ("show_teams", self.controller.on_menuitem_teams_activate),
                ("show_tags", self.controller.on_menuitem_tags_activate),
                ("show_about", self.controller.on_menuitem_about_activate),
            ))

    def __getitem__(self, key):
        return self.xml.get_widget(key)

    def do_about_show(self):
        about = gtk.AboutDialog()
        about.set_name("5-a-day Applet")
        about.set_comments(_("GNOME applet to commit bugs to the 5-a-day statistics project"))
        about.set_copyright("""Copyright (C) 2008 Markus Korn <thekorn@gmx.de>""")
        about.set_license(file('/usr/share/common-licenses/GPL-3').read())
        about.set_logo(self.about_icon)

        about.run()
        about.destroy()
        
    def do_teams_show(self):
        dialog = self['dialog_teams']
        self['dialog_teams_cancel'].connect("clicked", 
              lambda *args: dialog.response(gtk.RESPONSE_CANCEL))
        self['dialog_teams_apply'].connect("clicked", 
              lambda *args: dialog.response(gtk.RESPONSE_APPLY))

        dialog.show_all()
        response = dialog.run()
        dialog.hide()
        if response == gtk.RESPONSE_APPLY:
            return True
        return False
        
    def do_tags_show(self):
        dialog = self['dialog_tags']
        save = self.controller.tags_model.values
        self['dialog_tags_cancel'].connect("clicked", 
              lambda *args: dialog.response(gtk.RESPONSE_CANCEL))
        self['dialog_tags_apply'].connect("clicked", 
              lambda *args: dialog.response(gtk.RESPONSE_APPLY))
        self['entry_tags'].connect("key_press_event", self.add_tag, self.controller.tags_model)
        self['entry_tags'].set_text("")
        self['entry_tags'].set_tooltip_text(_("Press <Return> to add a tag"))
        self['treeview_tags'].connect("key_press_event", self.remove_tag, self.controller.tags_model)
        self['treeview_tags'].set_tooltip_text(_("Mark a tag and press <Del> to remove a tag"))

        dialog.show_all()
        response = dialog.run()
        dialog.hide()
        if response == gtk.RESPONSE_APPLY:
            x = self['entry_tags'].get_text()
            if x:
                self.controller.tags_model.add_tag(x)
                self['entry_tags'].set_text("")
            return True
        self.controller.tags_model.reset(save)
        return False
        
    def add_tag(self, widget, event, data):
        key = gtk.gdk.keyval_name (event.keyval)
        if key in ("KP_Enter", "Return"):
            text = widget.get_text()
            widget.set_text("")
            if text:
                data.add_tag(text)
        
    def remove_tag(self, widget, event, data):
        key = gtk.gdk.keyval_name (event.keyval)
        if key == "Delete":
            entry = self['treeview_tags'].get_selection()
            (model, entry) = entry.get_selected()
            if entry:
                data.remove_tag(entry)

    def do_wiki_open(self):
        fiveadayapplet.open_url("https://wiki.ubuntu.com/5-A-Day")

    def do_stats_open(self):
        fiveadayapplet.open_url("http://daniel.holba.ch/5-a-day-stats/")
        
    def do_login_show(self):
        dialog = self['dialog_login']
        login = self['entry_login']
        join_button = self['button_join']
        self['dialog_cancel'].connect("clicked", 
              lambda *args: dialog.response(gtk.RESPONSE_CANCEL))
        self['dialog_apply'].connect("clicked", 
              lambda *args: dialog.response(gtk.RESPONSE_APPLY))
        join_button.connect("clicked", 
              lambda *args: fiveadayapplet.open_url(join_button.get_uri()))

        dialog.show_all()
        response = dialog.run()
        dialog.hide()
        if response == gtk.RESPONSE_APPLY:
            return login.get_text()
        return None
        
    def change_tooltip(self, text):
        self.applet.set_tooltip_text(text)

    def setup_treeview(self, model):
        self['treeview_teams'].set_headers_visible(False)

        self['treeview_teams'].set_model(model)
        
        column0 = gtk.TreeViewColumn("Team", gtk.CellRendererText(), text=0)
        self['treeview_teams'].append_column(column0)
        
        renderer = gtk.CellRendererToggle()
        renderer.set_property('activatable', True)
        renderer.connect( 'toggled', self.team_toggled_cb, model )
        
        column1 = gtk.TreeViewColumn("", renderer )
        column1.add_attribute(renderer, "active", 1)

        self['treeview_teams'].append_column(column1)

    def setup_tags_treeview(self, model):
        self['treeview_tags'].set_headers_visible(False)

        self['treeview_tags'].set_model(model)
        
        column0 = gtk.TreeViewColumn("Tags", gtk.CellRendererText(), text=0)
        self['treeview_tags'].append_column(column0)
        
    def team_toggled_cb(self, cell, path, model):
        model[path][1] = not model[path][1]
        return
        
    def show_notification(self, title, text):
        n = pynotify.Notification(title, text, None)

        icon = gtk.gdk.pixbuf_new_from_file_at_size(self.icon_filename, 32, 32)
        n.set_icon_from_pixbuf(icon)
        n.attach_to_widget(self.applet)
        n.set_urgency(pynotify.URGENCY_LOW)
        n.add_action('close', "Hide", lambda *_: True)
        n.show()
